<?php

declare(strict_types=1);

namespace Drupal\collection\Plugin\Filter;

use Drupal\Component\Utility\Html;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\collection\Entity\CollectionContentInterface;
use Drupal\collection\Entity\CollectionInterface;
use Drupal\collection\Exception\MissingCollectionException;
use Drupal\filter\Attribute\Filter;
use Drupal\filter\FilterProcessResult;
use Drupal\filter\Plugin\FilterBase;
use Drupal\filter\Plugin\FilterInterface;
use Drupal\og\OgContextInterface;
use Drupal\og\OgGroupAudienceHelperInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Wraps glossary terms with links to their definition page.
 */
#[Filter(
  id: "collection_glossary",
  title: new TranslatableMarkup("Collection glossary"),
  type: FilterInterface::TYPE_TRANSFORM_IRREVERSIBLE,
  description: new TranslatableMarkup("Replaces glossary terms with their link version."),
)]
class Glossary extends FilterBase implements ContainerFactoryPluginInterface {

  public function __construct(
    array $configuration,
    string $plugin_id,
    $plugin_definition,
    protected readonly OgContextInterface $ogContext,
    protected readonly EntityTypeManagerInterface $entityTypeManager,
    protected readonly LoggerInterface $logger,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('og.context'),
      $container->get('entity_type.manager'),
      $container->get('logger.channel.collection')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function process($text, $langcode): FilterProcessResult {
    $result = new FilterProcessResult($text);

    $collection = $this->getCollection();
    // A collection context cannot be detected.
    if (!isset($collection)) {
      return $result;
    }

    /** @var \Drupal\Core\Cache\RefinableCacheableDependencyInterface $cache_metadata */
    [$replacements, $cache_metadata] = $this->getReplacementsMap($collection);

    // The reverse ksort will ensure that when, for any reason, there are terms
    // that overlap together, like "exchange", "exchange currency" and "exchange
    // platform", it is ensured, that terms with longer terms, are replaced
    // first and the shorter terms are coming last.
    krsort($replacements);

    // This collection has no glossary term entries.
    if (empty($replacements)) {
      return $result->addCacheableDependency($cache_metadata);
    }

    $escaped_replacements = array_map(function (string $replacement): string {
      return preg_quote($replacement, '/');
    }, array_keys($replacements));
    $pattern = '/\b(' . implode('|', $escaped_replacements) . ')\b/i';

    // First, do a bird-eye check for glossary terms so that we avoid a heavy
    // processing if the text contains no term.
    if (!preg_match($pattern, $text)) {
      return $result->addCacheableDependency($cache_metadata);
    }

    $link_only_first = $collection->getGlossarySettings()['link_only_first'];
    // Invalidate the cache when the collection glossary settings are changing.
    $cache_metadata->addCacheableDependency($collection->get('collection_settings')->entity);

    $document = Html::load($text);
    $text_nodes = (new \DOMXPath($document))->evaluate("//text()");
    foreach ($text_nodes as $text_node) {
      // Avoid adding nested links.
      if ($this->isLinkText($text_node)) {
        continue;
      }

      $text_parts = preg_split($pattern, $text_node->nodeValue, -1, PREG_SPLIT_DELIM_CAPTURE);
      $parent_node = $text_node->parentNode;

      foreach ($text_parts as $text_part) {
        $text_part_lowercased = \mb_strtolower($text_part);
        if (!isset($replacements[$text_part_lowercased])) {
          $parent_node->insertBefore($document->createTextNode($text_part), $text_node);
          continue;
        }

        // @todo Consider replacing the link with a JavaScript tooltip.
        $link = $document->createElement('a', $text_part);
        $link->setAttribute('href', $replacements[$text_part_lowercased]['url']);
        $link->setAttribute('class', 'glossary-term');
        $link->setAttribute('title', $replacements[$text_part_lowercased]['summary']);
        $parent_node->insertBefore($link, $text_node);

        // If this collection was configured to replace only the first
        // occurrence of the term, remove this match from replacements list.
        if ($link_only_first) {
          unset($replacements[$text_part_lowercased]);
        }
      }
      $parent_node->removeChild($text_node);
    }

    return $result
      ->setProcessedText($document->saveHTML())
      ->addCacheableDependency($cache_metadata);
  }

  /**
   * Finds-out if a piece of text is inside a HTML link.
   *
   * @param \DOMText $text_node
   *   The text node to be checked.
   *
   * @return bool
   *   If the given text node is inside a HTML link.
   */
  protected function isLinkText(\DOMText $text_node): bool {
    $node = $text_node;
    do {
      $node = $node->parentNode;
      if (!$node->parentNode) {
        return FALSE;
      }
    } while ($node->tagName !== 'a');
    return TRUE;
  }

  /**
   * Builds and returns a replacements map.
   *
   * @param \Drupal\collection\Entity\CollectionInterface $collection
   *   The collection for which to return the replacements map.
   *
   * @return array
   *   An indexed array (tuple) with two values:
   *     0: An associative array keyed by the glossary term or synonym. The
   *        values are arrays with two keys:
   *        - url: The glossary term URL.
   *        - summary: A summary to be used as tooltip.
   *     1: An object containing the cacheable metadata.
   */
  protected function getReplacementsMap(CollectionInterface $collection): array {
    // Make sure the filter cache invalidates when a new glossary term is added
    // in this collection.
    $cache_metadata = (new CacheableMetadata())
      ->addCacheTags(
        Cache::buildTags('og-group-content', $collection->getCacheTagsToInvalidate())
      );

    $map = [];
    $node_storage = $this->entityTypeManager->getStorage('node');
    $nids = $node_storage->getQuery()->accessCheck(FALSE)
      ->condition('type', 'glossary')
      ->condition(OgGroupAudienceHelperInterface::DEFAULT_FIELD, $collection->id())
      ->condition('status', TRUE)
      ->execute();

    if ($nids) {
      /** @var \Drupal\collection\Entity\GlossaryTermInterface $glossary */
      foreach ($node_storage->loadMultiple($nids) as $glossary) {
        $link = [
          'url' => $glossary->toUrl()->toString(),
          'summary' => $glossary->getSummary(),
        ];

        // Ensure case-insensitive search.
        $label = \mb_strtolower($glossary->label());
        $map[$label] = $link;
        // Link also the synonyms, if any.
        foreach ($glossary->getSynonyms() as $synonym) {
          // Ensure synonym case-insensitive search.
          $synonym = \mb_strtolower($synonym);
          if (!isset($map[$synonym])) {
            $map[$synonym] = $link;
          }
        }

        // When this glossary node is changing, invalidate the filter cache.
        $cache_metadata->addCacheableDependency($glossary);
      }
    }

    return [$map, $cache_metadata];
  }

  /**
   * Retrieves the collection from the request context.
   *
   * This is not ideal but text filters have no direct way to retrieve the
   * context they belong to. We do not have access to the field that contains
   * the text.
   *
   * @return \Drupal\collection\Entity\CollectionInterface|null
   *   The collection, or NULL if no collection could be derived from the
   *   context.
   *
   * @see https://www.drupal.org/project/drupal/issues/226963
   */
  protected function getCollection(): ?CollectionInterface {
    if ($group = $this->ogContext->getGroup()) {
      // Other kind of group? Maybe a solution which is collection content?
      if ($group instanceof CollectionContentInterface) {
        try {
          $group = $group->getCollection();
        }
        catch (MissingCollectionException $e) {
          // The content is orphaned. Log an error but allow the request to
          // continue, this is not fatal.
          $this->logger->error('Collection could not be retrieved from entity of type %type and ID %id', [
            '%type' => $group->getEntityTypeId(),
            '%id' => $group->id(),
          ]);
        }
      }
    }

    if ($group instanceof CollectionInterface) {
      return $group;
    }

    return NULL;
  }

}
