<?php

declare(strict_types=1);

namespace Drupal\joinup_solution\Plugin\Validation\Constraint;

use Drupal\Core\DependencyInjection\AutowireTrait;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\joinup_solution\Entity\SolutionInterface;
use Drupal\joinup_solution\SolutionTitleDuplicateHelperInterface;
use Symfony\Component\DependencyInjection\Attribute\Autowire;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;

/**
 * Validates that a field is unique for the given entity type within a bundle.
 *
 * This is the validator for the UniqueSolutionTitle.
 *
 * The following checks make sure that a solution must have a unique title among
 * within their collections.
 *
 * @see \Drupal\joinup_solution\Plugin\Validation\Constraint\UniqueSolutionTitle
 */
class UniqueSolutionTitleValidator extends ConstraintValidator implements ContainerInjectionInterface {

  use AutowireTrait;

  public function __construct(
    #[Autowire(service: 'joinup_solution.title_duplicate_helper')]
    protected readonly SolutionTitleDuplicateHelperInterface $solutionTitleHelper,
  ) {}

  /**
   * {@inheritdoc}
   */
  public function validate($items, Constraint $constraint): void {
    if (!$item = $items->first()) {
      return;
    }

    /** @var \Drupal\joinup_solution\Entity\SolutionInterface $entity */
    $entity = $items->getEntity();
    if (!$entity instanceof SolutionInterface) {
      // In nodes, the title is a base field and the constraint cannot be set
      // per bundle. So, escape early if the entity is not a solution.
      return;
    }

    // Either the solution's title is not unique within affiliation, or the
    // solution title has duplicates with other solutions, all without
    // affiliation. In the first case ::titleIsUniqueWithinAffiliation() returns
    // FALSE, in the latter NULL.
    // @see \Drupal\joinup_solution\SolutionTitleDuplicateHelperInterface::titleIsUniqueWithinAffiliation()
    if (!$this->solutionTitleHelper->titleIsUniqueWithinAffiliation($entity)) {
      $this->context->addViolation($constraint->message, [
        '%value' => $item->value,
        '@entity_type' => $entity->getEntityType()->getSingularLabel(),
        '@field_name' => $items->getFieldDefinition()->getLabel(),
      ]);
    }
  }

}
